/* 

Allan Spale
CS 488
Assignment 1

This is the main file for the running the program and rendering the data
from the earthquake and landscape files.

*/

#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <GL/glut.h>
#include "DataManagement.h"
#include "CalendarDate.h"
#include "Transformation.h"
#include "Point.h"


//--------------------------WRAPPER FUNCTIONS-------------------------------//
// Originally, the OpenGL and GLUT functions were originally contained in
// the class named DataView.  However, because of compiling errors with
// being able to convert void to void (__cdecl *), these "local" functions
// will be used to call the relevant DataView public function.  Also, it is
// necessary to declare a global variable in order to call "actual" function
// from within the class.


// global vars
enum Playback { PBK_STOP, PBK_PLAY, PBK_PAUSE };

Point ptLastMouseLocation;
Transformation trTransform;
float fMinimumMagnitude, fMaximumDepth;
long lEarthquakeDataSize, lLandscapeDataSize, lStartingIndex, lEndingIndex;
int iCityDataSize, iMainWindowID, iTopDownWindowID, iSLookingNWindowID, 
	iELookingWWindowID;
Playback pbkAnimation;
CityLocation *clpCityInfo;
LandscapeLocation *llpLandscapeInfo;
QuakeData *qdpQuakeInfo;
CalendarDate cdStartDate, cdEndDate;


//-------------------------NON-OpenGL FUNCTIONS------------------------------//
void zMoveDatasetWindowForward() {
	if ( lEndingIndex >= lEarthquakeDataSize ) {
		pbkAnimation = PBK_PAUSE;
	}
	else {
		cdEndDate.zGoForwardInterval();
		cdStartDate.zGoForwardInterval();

		// get earliest occurrence of the "starting date"
		while ( ( qdpQuakeInfo[ lStartingIndex ].cdDate.bIsLessThan( cdStartDate ) == true )
			&&  ( lStartingIndex < lEndingIndex ) ) {
			lStartingIndex++;
		}

		// get next occurrence of the "starting date"
		while ( ( qdpQuakeInfo[ lEndingIndex ].cdDate.bIsLessThan( cdEndDate ) == true )
			&&  ( lEndingIndex < lEarthquakeDataSize ) ) {
			lEndingIndex++; 
		}

		// get last duplicate value of the "ending date"
		while ( ( qdpQuakeInfo[ lEndingIndex ].cdDate.bIsEqualTo( cdEndDate ) == true )
			&&  ( lEndingIndex < lEarthquakeDataSize ) ) {
			lEndingIndex++; 
		}
	}

}


// step backward through the earthquake data; cannot animate backward
void zMoveDatasetWindowBackward() {
	if ( lStartingIndex > 0 ) {
		
		if ( lEndingIndex >= lEarthquakeDataSize ) {
			lEndingIndex = lEarthquakeDataSize - 1;
		}

		cdEndDate.zGoBackwardInterval();
		cdStartDate.zGoBackwardInterval();

		// get last, earliest occurrence of the "ending date"
		while ( ( qdpQuakeInfo[ lEndingIndex ].cdDate.bIsGreaterThan( cdEndDate ) == true )
			&&  ( lEndingIndex > 0 ) ) {
			lEndingIndex--; 
		}

		// get the most recent occurrence of "starting date"
		while ( ( qdpQuakeInfo[ lStartingIndex ].cdDate.bIsGreaterThan( cdStartDate ) == true )
			&&  ( lStartingIndex > 0 ) ) {
			lStartingIndex--;
		}
		// get the earliest occurrence of "starting date"
		while ( ( qdpQuakeInfo[ lStartingIndex ].cdDate.bIsEqualTo( cdStartDate ) == true )
			&&  ( lStartingIndex > 0 ) ) {
			lStartingIndex--;
		}
	}
}


// find the first occurrence of date (looking from the start of the file)
// in the earthquake data file and return the position of the
// occurrence in the file 
long cdGetLocationOfMonthInEarthquakeData( CalendarDate cdDate ) {
	long lIndex;

	lIndex = lStartingIndex;

	while ( ( cdDate.bIsGreaterThan( qdpQuakeInfo[ lIndex ].cdDate ) )  &&
			( lIndex < lEarthquakeDataSize ) ) {
		lIndex++;
	}

	return lIndex;
}
		

// process ASCII-based keypresses
void zProcessASCIIKeyboard( unsigned char ucKeypress, 
						    int iX, int iY ) {
	bool bUpdateMagnitudeFilter;
	float fPreCalculation;

	bUpdateMagnitudeFilter = true;

	switch( ucKeypress ) {


		// escape key (ASCII=27) exit program key
		case 27:
			cout << "Exiting..." << endl << endl;
			exit(0);

		// magnitude filter keys...subtract 48 from character value to 
		// obtain integer equivalent
		case '0':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
			fMinimumMagnitude = ucKeypress - 48;
			break;


		// zoom keys...'A'/'a': zoom in,
		//			   'Z','z': zoom out
		case 'a':
		case 'A':
			trTransform.fScaleValue += trTransform.fScaleIncrement;
			break;

		case 'z':
		case 'Z':
			fPreCalculation = trTransform.fScaleValue - trTransform.fScaleIncrement;

			if ( fPreCalculation > 0.001 ) {
				trTransform.fScaleValue = fPreCalculation;
			}
			break;


		// time-travel keys...'G'/'g': time goes forward
		//                    'H'/'h': time goes backward
		case 'g':
		case 'G':
			if ( pbkAnimation == PBK_STOP ) {
				pbkAnimation = PBK_PAUSE;
				cdStartDate.zSetCalendarDate( qdpQuakeInfo[ lStartingIndex ].cdDate );
				cdEndDate.zSetCalendarDate( cdStartDate );
				cdEndDate.zGoForward30Days();
				lEndingIndex = cdGetLocationOfMonthInEarthquakeData( cdEndDate );
			}
			else if ( pbkAnimation == PBK_PLAY ) {
				pbkAnimation = PBK_PAUSE;
			}
			zMoveDatasetWindowForward();
			break;

		case 'h':
		case 'H':
			if ( pbkAnimation == PBK_STOP ) {
				pbkAnimation = PBK_PAUSE;
				cdStartDate.zSetCalendarDate( qdpQuakeInfo[ lStartingIndex ].cdDate );
				cdEndDate.zSetCalendarDate( cdStartDate );
				cdEndDate.zGoForward30Days();
				lEndingIndex = cdGetLocationOfMonthInEarthquakeData( cdEndDate );
			}
			else if ( pbkAnimation == PBK_PLAY ) {
				pbkAnimation = PBK_PAUSE;

			}
			zMoveDatasetWindowBackward();

			break;


		// depth exaggeration keys...'X'/'x': increase exaggeration
		//						  ...'S'/'s': decrease exaggeration
		case 'X':
		case 'x':
			trTransform.fDepthExaggerationValue += 
				trTransform.fDepthExaggerationIncrement;
			break;

		case 'S':
		case 's':
			fPreCalculation = trTransform.fDepthExaggerationValue
				- trTransform.fDepthExaggerationIncrement;

			if ( fPreCalculation > 0.001 ) {
				trTransform.fDepthExaggerationValue = fPreCalculation;
			}
			break;


		// data view keys...'O'/'o': show all earthquakes
		//               ...'P'/'p': playback earthquakes
		case 'o':
		case 'O':
			pbkAnimation = PBK_STOP;
			lStartingIndex = 0;
			lEndingIndex = lEarthquakeDataSize - 1;
			cdStartDate.zSetCalendarDate( qdpQuakeInfo[ lStartingIndex ].cdDate );
			cdEndDate.zSetCalendarDate( qdpQuakeInfo[ lEndingIndex ].cdDate );
			break;

		case 'p':
		case 'P':
			if ( pbkAnimation == PBK_PAUSE ) {
				// pick up from paused location
				pbkAnimation = PBK_PLAY;
			}
			else if ( pbkAnimation == PBK_STOP ) {
				// start from the beginning
				cdStartDate.zSetCalendarDate( qdpQuakeInfo[ lStartingIndex ].cdDate );
				cdEndDate.zSetCalendarDate( cdStartDate );
				cdEndDate.zGoForward30Days();
				lEndingIndex = cdGetLocationOfMonthInEarthquakeData( cdEndDate );
				pbkAnimation = PBK_PLAY;
			}
			// else ignore messge if already playing
			break;

		default:
			break;
	}
}


/*************************************************************
/* output code by Mark Kilgard */
 
void output(float x, float y, char *string)
{
	int len, i;
	
	glRasterPos2f(x, y);
	len = (int) strlen(string);

	for (i = 0; i < len; i++) {
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, string[i]);
	}
}

// Draw the city name on the landscape and "scale" the font of the city
// name by choosing the appropriate bitmapped font
void outputCityText( float fX, float fY, float fZ, char *string ) {
	int len, i, iCurrentWindow;

	glColor3f( 0.0, 0.8, 0.0 );  // light green
	glBegin( GL_POLYGON );
		glVertex3f( fX + 0.1, fY + 0.1,	0 );
		glVertex3f( fX - 0.1, fY + 0.1, 0 );
		glVertex3f( fX - 0.1, fY - 0.1, 0 );
		glVertex3f( fX + 0.1, fY - 0.1, 0 );
	glEnd();

	glColor3f( 1,1,1  );
	glRasterPos3f( fX+0.1, fY, fZ );

	len = (int) strlen(string);
	iCurrentWindow = glutGetWindow();


	// Only the "main window" has scaling fonts; the smaller windows only
	// have fixed size fonts; repeated loops were used because I thought
	// that having a switch statement inside a loop would be more costly
	// to the rendering process instead of having separate loops that
	// can execute as quickly as possible.

	if ( iCurrentWindow == iMainWindowID ) {

		if ( trTransform.fScaleValue > 3 ) {
			for (i = 0; i < len; i++) {
				glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, string[i]);
			}
		}
		else if ( trTransform.fScaleValue >= 1.25 ) {
			for (i = 0; i < len; i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, string[i]);
			}
		}
		else if ( trTransform.fScaleValue >= 0.7 ) {
			for (i = 0; i < len; i++) {
				glutBitmapCharacter(GLUT_BITMAP_9_BY_15, string[i]);
			}
		}
		else if ( trTransform.fScaleValue >= 0.5 ) {
			for (i = 0; i < len; i++) {
				glutBitmapCharacter(GLUT_BITMAP_8_BY_13, string[i]);
			}
		}
		else if ( trTransform.fScaleValue >= 0.25 ) {
			for (i = 0; i < len; i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, string[i]);
			}
		}
		else if ( trTransform.fScaleValue >= 0.0 ) {
			for (i = 0; i < len; i++) {
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, string[i]);
			}
		}
	}
	else {
		// fixed font size for smaller windows
		for (i = 0; i < len; i++) {
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, string[i]);
		}
	}
}


// process the non-ASCII based keypresses
void zProcessNonASCIIKeyboard( int iKeypress, int iX, int iY ) {
	switch( iKeypress ) {

		// pan keys...LEFT ARROW: pan left
		//			  RIGHT ARROW: pan right
		//			  UP ARROW: pan up  (note: max latitude is at top of window)
		//			  DOWN ARROW: pan down (note: min latitude is at bottom of window)
		
		case GLUT_KEY_LEFT:
			trTransform.xyzTranslateVector.fX += 
				trTransform.fTranslateIncrement;
			break;

		case GLUT_KEY_RIGHT:
			trTransform.xyzTranslateVector.fX -= 
				trTransform.fTranslateIncrement;
			break;

		case GLUT_KEY_UP:
			trTransform.xyzTranslateVector.fY -= 
				trTransform.fTranslateIncrement;
			break;

		case GLUT_KEY_DOWN:
			trTransform.xyzTranslateVector.fY += 
				trTransform.fTranslateIncrement;
			break;

		// used to restore the initial configuration of the main window
		case GLUT_KEY_HOME:
			trTransform.fScaleValue = 1.0;
			trTransform.xyzTranslateVector.fX = 0.0;
			trTransform.xyzTranslateVector.fY = 0.0;
			trTransform.xyzTranslateVector.fZ = 0.0;
			trTransform.fDepthExaggerationValue = 8.0;
			trTransform.xyzRotateVector.fX = 0.0;
			trTransform.xyzRotateVector.fY = 0.0;
			trTransform.xyzRotateVector.fZ = 0.0;
			break;

		default:
			break;
	}	
}


// mouse pressing function
void zMouseButtonPress( int iButton, int iState, int iX, int iY ) {
	if ( iButton == GLUT_LEFT_BUTTON ) {
		switch ( iState ) {
			case GLUT_UP:
				break;

			case GLUT_DOWN:
				ptLastMouseLocation.fX = iX;
				ptLastMouseLocation.fY = iY;
				break;
		}
	}
}


// mouse motion function
void zMouseMotion( int iX, int iY ) {
	float fXRotation, fYRotation;
	float fXChange, fYChange;

		fXChange = ptLastMouseLocation.fX - iX;
		fYChange = ptLastMouseLocation.fY - iY;
		fXRotation = ( fXChange ) * trTransform.fRotateIncrement;
		fYRotation = ( fYChange ) * trTransform.fRotateIncrement;
		trTransform.xyzRotateVector.fY += fXRotation;
		trTransform.xyzRotateVector.fX += fYRotation;

		// try to keep odd angles from existing...even though it does not
		// really matter with the result of the transformations
		if ( trTransform.xyzRotateVector.fX > 360 ) {
			trTransform.xyzRotateVector.fX -= 360;
		}
		else if ( trTransform.xyzRotateVector.fX < -360 ) {
			trTransform.xyzRotateVector.fX += 360;
		}

		if ( trTransform.xyzRotateVector.fY > 360 ) {
			trTransform.xyzRotateVector.fY -= 360;
		}
		else if ( trTransform.xyzRotateVector.fY < -360 ) {
			trTransform.xyzRotateVector.fY += 360;
		}

		ptLastMouseLocation.fX = iX;
		ptLastMouseLocation.fY = iY;
}


// update the visible data range for the animation
void zUpdateData( int iValue ) {
	// use for animation sequence
	if ( pbkAnimation == PBK_PLAY ) {
		zMoveDatasetWindowForward();
	}

	// keep fixed to maintain stable frame value
    glutTimerFunc(50, zUpdateData, 1);
}


// setup window for rendering
void zInit() {
	// set up the logical graphics space
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
		
	/* clear the screen to the clear colour */
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// check for invalid colormap
	if ( glutGet( GLUT_WINDOW_COLORMAP_SIZE ) != 0 )
	{
		cout << "Error: Invalud RGBA colormap size" << endl;
		exit( 1 );
	}
}


// visualize the data
void zPaint() {
	long lIndex;
	float fMagnitude, fPlotHalfSize, fPlotHalfWidthRatio;
	XYZValue xyzLandscape, xyzEarthquake;


	// plot earthquake data
	for ( lIndex = lStartingIndex; lIndex < lEndingIndex; lIndex++ ) {

		fMagnitude = qdpQuakeInfo[ lIndex ].fMagnitude;

		if ( fMagnitude >= fMinimumMagnitude ) {
			// determine color
			glColor3f( 1.0, qdpQuakeInfo[ lIndex ].fMagnitudeGreenColor, 0.0 );

			// plot

			// scale the depth of the earthquake according to the "exaggerated"
			// depth value
			xyzEarthquake.fZ = ( qdpQuakeInfo[ lIndex ].qlLocation.xyzLocation.fZ * 
				trTransform.fDepthExaggerationValue ) / fMaximumDepth;

			xyzEarthquake.fY = qdpQuakeInfo[ lIndex ].qlLocation.xyzLocation.fY;
			xyzEarthquake.fX = qdpQuakeInfo[ lIndex ].qlLocation.xyzLocation.fX;

			fPlotHalfWidthRatio = .026;
			fPlotHalfSize = fMagnitude * fPlotHalfWidthRatio;
			
			glBegin( GL_LINES );
				// X line
				glVertex3f( xyzEarthquake.fX - fPlotHalfSize, xyzEarthquake.fY,
					xyzEarthquake.fZ );
				glVertex3f( xyzEarthquake.fX + fPlotHalfSize, xyzEarthquake.fY, 
					xyzEarthquake.fZ );

				// Y line
				glVertex3f( xyzEarthquake.fX, xyzEarthquake.fY - fPlotHalfSize,
					xyzEarthquake.fZ );
				glVertex3f( xyzEarthquake.fX , xyzEarthquake.fY + fPlotHalfSize,
					xyzEarthquake.fZ );

				// Z line
				glVertex3f( xyzEarthquake.fX, xyzEarthquake.fY,  
					xyzEarthquake.fZ - fPlotHalfSize );
				glVertex3f( xyzEarthquake.fX , xyzEarthquake.fY,
					xyzEarthquake.fZ + fPlotHalfSize );


			glEnd();
		} // end magnitude filter condition
	}
	
	// draw landscape data with polygon separation
	glColor3f( 1.0, 1.0, 1.0 );  //white
	
	glBegin( GL_LINE_LOOP );  // start with this by default

		for ( lIndex = 0; lIndex < lLandscapeDataSize; lIndex++ ) {
			glVertex3f( llpLandscapeInfo[ lIndex ].fLongitude, 
				llpLandscapeInfo[ lIndex ].fLatitude, 0 );

			if ( llpLandscapeInfo[ lIndex ].bLastPoint == true ) {
				// end the current "line loop" and begin a new "line loop"
				glEnd();
				glBegin( GL_LINE_LOOP );
			}

		}

	glEnd();  // add extra for last polygon point


	// draw city data
	for ( lIndex = 0; lIndex < iCityDataSize; lIndex++ ) {
		outputCityText( clpCityInfo[ lIndex ].llLocation.fLongitude,
						clpCityInfo[ lIndex ].llLocation.fLatitude,
						0, clpCityInfo[ lIndex ].c1dCityName );
	}	
}


// draw main window
void zMainWindowPaint() {
	char c1dMagnitudeDisplay[ 15 ], cpDateDisplay[ 24 ];

	glutSetWindow( iMainWindowID );
	//zInit();
	glPushMatrix();   // identity matrix pushed
		gluPerspective(60, 1, 1, 1000);
		gluLookAt(0,-10, 15,    0, 0, 0,      0, 1, 0);
		
		glPushMatrix();  // viewpoint setup pushed

			glRotatef( trTransform.xyzRotateVector.fX, 1, 0, 0 );
			glRotatef( trTransform.xyzRotateVector.fY, 0, 1, 0 );

			glTranslatef( trTransform.xyzTranslateVector.fX, 
				trTransform.xyzTranslateVector.fY, 
				trTransform.xyzTranslateVector.fZ );
			
			glScalef( trTransform.fScaleValue, 
				trTransform.fScaleValue, trTransform.fScaleValue );

			zPaint();

		glPopMatrix();  // original viewpoint setup popped

		// output text display
		sprintf( c1dMagnitudeDisplay, "Magnitude >= %1d", (int)fMinimumMagnitude );
		sprintf( cpDateDisplay, "%s - %s", cdStartDate.cpDisplayDate(),
			cdEndDate.cpDisplayDate() );
	
		// black background for "status line" to imporve readability
		glColor3f( 0.0, 0.0, 0.0 );
		glBegin( GL_QUADS );
			glVertex2f( -10.5, -8.3 );
			glVertex2f( 10.5, -8.3 );
			glVertex2f( 10.5, -10.0 );
			glVertex2f( -10.5, -10.0 );
		glEnd();

		glColor3f( 1.0, 0.0, 0.0 );
		output( -7.2, -8.7, c1dMagnitudeDisplay );
		output( 2.5, -8.7, cpDateDisplay );

		glutSwapBuffers();

	glPopMatrix();
	glutPostRedisplay();
}


// draw window of top-down view
void zTopDownWindowPaint() {
	glutSetWindow( iTopDownWindowID );
	zInit();
	glPushMatrix();   // identity matrix pushed
	gluPerspective(60, 1, 1, 1000);
	gluLookAt(0,0,15,    0, 0, 0,      0, 1, 0);
	glPushMatrix();  // viewpoint setup pushed
	zPaint();
	glutSwapBuffers();
	glPopMatrix();
	glPopMatrix();
	glutPostRedisplay();
}


// draw window of south looking north view
void zSLookingNWindowPaint() {
	glutSetWindow( iSLookingNWindowID );
	zInit();
	glPushMatrix();   // identity matrix pushed
		gluPerspective(60, 1, 1, 1000);
		gluLookAt(0, -24, .00001,    0, 0, 0,      0, 1, 0);
	
		glPushMatrix();  // viewpoint setup pushed
			zPaint();
			glutSwapBuffers();
		glPopMatrix();
	
	glPopMatrix();
	glutPostRedisplay();
}


// draw window of east looking west view
void zELookingWWindowPaint() {
	glutSetWindow( iELookingWWindowID );
	zInit();
	glPushMatrix();   // identity matrix pushed
		gluPerspective(60, 1, 1, 1000);
		gluLookAt(21, 0, 0.0001,    0, 0, 0,      -1, 0, 0);
		
		glPushMatrix();  // viewpoint setup pushed
			zPaint();
			glutSwapBuffers();
		glPopMatrix();

	glPopMatrix();
	glutPostRedisplay();
}



// MAIN PROGRAM
int main( int argc, char **argv ) {
	bool bContinue;
	DataManagement dmInfoStore;

	// set default values
	trTransform.fScaleIncrement = 0.07;
	trTransform.fScaleValue = 1.0;
	trTransform.fRotateIncrement = 0.4;
	trTransform.xyzRotateVector.fX = 0.0;
	trTransform.xyzRotateVector.fY = 0.0;
	trTransform.xyzRotateVector.fZ = 0.0;
	trTransform.fTranslateIncrement = 0.25;
	trTransform.xyzTranslateVector.fX = 0.0;
	trTransform.xyzTranslateVector.fY = 0.0;
	trTransform.xyzTranslateVector.fZ = 0.0;
	trTransform.fDepthExaggerationValue = 8.0;
	trTransform.fDepthExaggerationIncrement = 0.3; 
	pbkAnimation = PBK_STOP; 


	
	bContinue = false;
	
	if ( argc < 4 ) {
		cout << "'3dquakes' must have two valid filenames as parameters" << endl;
		cout << "usage: 3dquakes landscape.txt data.txt cities.txt" << endl << endl;
	}
	else {
		bContinue = 
			//dmInfoStore.bReadDataFiles( "C:\\Allan-Folder\\cs488\\asgn2\\3japanlatlon2.txt",
			//							"C:\\Allan-Folder\\cs488\\asgn2\\3japanquake.txt",
			//							"C:\\Allan-Folder\\cs488\\asgn2\\3japancity.txt" );
			dmInfoStore.bReadDataFiles( argv[ 1 ], argv[ 2 ], argv[ 3 ] );
	}

	if ( bContinue == true ) {

		llpLandscapeInfo = dmInfoStore.llpGetLandscapeData();
		qdpQuakeInfo = dmInfoStore.qdpGetEarthquakeData();
		clpCityInfo = dmInfoStore.clpGetCityData();

		fMaximumDepth = -dmInfoStore.fGetMaximumDepth();  //use negative depth
		lEarthquakeDataSize = dmInfoStore.lGetQuakeDataSize();
		lLandscapeDataSize = dmInfoStore.lGetLandscapeDataSize();
		iCityDataSize = dmInfoStore.iGetCityDataSize();
		
		lStartingIndex = 0;
		lEndingIndex = lEarthquakeDataSize - 1;
		
		cdStartDate.zSetCalendarDate( qdpQuakeInfo[ lStartingIndex ].cdDate );
		cdEndDate.zSetCalendarDate( qdpQuakeInfo[ lEndingIndex ].cdDate );

		// CALCULATE DAYS TO ELAPSE
		// Since 1 minute is time for animation and 50ms elapse per frame...
		// 1000 ms / 50 ms = 20 updates/sec
		// 20 updates/sec * 60 sec = 1200 updates
		//
		// roughly estimate the number of days in the earthquake data
		// by counting number of years

		// NOTE: For assignment 2, forcing display refreshes has increased
		// the frame rate (or so it seems)... changing 1200 to 2400

		CalendarDate::szSetIntervalOfDays( (int) (
			CalendarDate::slCountDaysInInterval( 
				cdStartDate, cdEndDate ) / 1800 ) );  // /2

		//----------------SETUP OpenGL and GLUT----------\\
		// process any GLUT command Line options
		glutInit( &argc, argv );

		// create output window
		glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );

		//--------------- MAIN WINDOW-------------------\\
		// set up the physical window size
		glutInitWindowSize( 650, 650 );
		glutInitWindowPosition( 0, 0 );

		// set the name of the window and try to create it
		iMainWindowID = glutCreateWindow("CS 488 - HW 2");
		
		// specify function for ASCII keyboard command processing
		glutKeyboardFunc( zProcessASCIIKeyboard );

		// specify function for non-ASCII keyboard command processing
		glutSpecialFunc( zProcessNonASCIIKeyboard );

		// specify function for mouse button press
		glutMouseFunc( zMouseButtonPress );

		// specify function for mouse motion
		glutMotionFunc( zMouseMotion );

		// specify function for display
		glutDisplayFunc( display );

		// specify timer for updating the scene
		glutTimerFunc( 50, zUpdateData, 1 );


		//------------------- TOP-DOWN WINDOW -------------------------\\
		// set up the physical window size
		glutInitWindowSize( 200, 200 );
		glutInitWindowPosition( 660, 0 );

		// set the name of the window and try to create it
		iTopDownWindowID = glutCreateWindow("top down");

		// specify function for display
		glutDisplayFunc( zTopDownWindowPaint );

		// specify function for execution during idle time
		glutIdleFunc( zTopDownWindowPaint );

		// specify timer for updating the scene
		glutTimerFunc( 50, zUpdateData, 1 );


		//---------------- S LOOKING N WINDOW ------------------------\\
		// set up the physical window size
		glutInitWindowSize( 200, 200 );
		glutInitWindowPosition( 660, 230 );

		// set the name of the window and try to create it
		iSLookingNWindowID = glutCreateWindow("S looking N");

		// specify function for display
		glutDisplayFunc( zSLookingNWindowPaint );

		// specify function for execution during idle time
		glutIdleFunc( zSLookingNWindowPaint );

		// specify timer for updating the scene
		glutTimerFunc( 50, zUpdateData, 1 );


		//----------------- E LOOKING W WINDOW----------------------\\
		// set up the physical window size
		glutInitWindowSize( 200, 200 );
		glutInitWindowPosition( 660, 455 );

		// set the name of the window and try to create it
		iELookingWWindowID = glutCreateWindow("E looking W");
		//	glutCreateSubWindow( iMainWindowID,650, 500,250, 250);

		// specify function for display
		glutDisplayFunc( zELookingWWindowPaint );

		// specify function for execution during idle time
		glutIdleFunc( zELookingWWindowPaint );

		// specify timer for updating the scene
		glutTimerFunc( 50, zUpdateData, 1 );

 
		glutMainLoop();
		
		// delete pointers
		delete [] clpCityInfo;
		delete [] llpLandscapeInfo;
		delete [] qdpQuakeInfo;

		return 0; 
	}
	else {
		// error messages should have already been printed by this point
		return -1;
	}
}
