/* 

Allan Spale
CS 488
Assignment 2

This class will manage the calculation, comparison, and string coversion
of calendar dates.  In order to increase efficiency (i.e. no function
calls for every day, month, and year access), the date class member 
vairables are made public instead of being private.  The interval class 
variable is used to step forward and backward through time.

*/

#include "CalendarDate.h" 

// maximum days in a month constants (note, no month 0)
const int CalendarDate::I1D_NUMBER_OF_DAYS_IN_MTH[ 13 ] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

// default interval value
int CalendarDate::iInterval = 1;


CalendarDate::CalendarDate() {
	iYear = 1;
	iMonth = 1;
	iDay = 1;
	cpStringDate = new char[ 11 ];
	zConvertToStringDate();
}


// 30 days is the "rough equivalent" of one month
void
CalendarDate::zGoForward30Days() {
	int iOldInterval;

	iOldInterval = iInterval;
	iInterval = 30;
	zGoForwardInterval();
	iInterval = iOldInterval;
}
	

// Add the interval value to "this" date and recalculate the appropriate
// new date.
void
CalendarDate::zGoForwardInterval() {
	int iFirstMonthDays, iIntervalCountdown;

	iIntervalCountdown = iInterval;
	iFirstMonthDays = I1D_NUMBER_OF_DAYS_IN_MTH[ iMonth ] - iDay;
	iIntervalCountdown -= iFirstMonthDays;


	if ( iIntervalCountdown <= 0 ) {
		iDay += iInterval;
	}
	else {
		iMonth++;

		if ( iMonth > 12 ) {
			iMonth = 1;
			iYear++;
		}

		while ( iIntervalCountdown > I1D_NUMBER_OF_DAYS_IN_MTH[ iMonth ] ) {
			iIntervalCountdown -= I1D_NUMBER_OF_DAYS_IN_MTH[ iMonth ];
			iMonth++;

			if ( iMonth > 12 ) {
				iMonth = 1;
				iYear++;
			}
		}

		iDay = iIntervalCountdown;
	}

	zConvertToStringDate();
}


// Subtract the interval value to "this" date and recalculate the 
// appropriate new date.
void
CalendarDate::zGoBackwardInterval() {
	int iIntervalCountdown;

	iIntervalCountdown = iInterval;
	iIntervalCountdown -= iDay;

	if ( iIntervalCountdown < 0 ) {
		iDay -= iInterval;
	}
	else {
		iMonth--;

		if ( iMonth < 1 ) {
			iMonth = 12;
			iYear--;
		}

		while ( iIntervalCountdown >= I1D_NUMBER_OF_DAYS_IN_MTH[ iMonth ] ) {
			iIntervalCountdown -= I1D_NUMBER_OF_DAYS_IN_MTH[ iMonth ];
			iMonth--;

			if ( iMonth < 1 ) {
				iMonth = 12;
				iYear--;
			}
		}

		iDay = I1D_NUMBER_OF_DAYS_IN_MTH[ iMonth ] - iIntervalCountdown;
	}

	zConvertToStringDate();
}



void
CalendarDate::szSetIntervalOfDays( int iNumberOfDays ) {
	iInterval = iNumberOfDays;
}


// make the following comparison: is this.date < parameter.date
bool
CalendarDate::bIsLessThan( CalendarDate cdDate ) {
	if ( iYear < cdDate.iYear ) {
		return true;
	}
	else if ( iYear == cdDate.iYear ) {
		if ( iMonth < cdDate.iMonth ) {
			return true;
		}
		else if ( iMonth == cdDate.iMonth ) {
			if ( iDay < cdDate.iDay ) {
				return true;
			}
			else {
				// iDay >= cdDate.iDay
				return false;
			}
		}
		else {
			// iMonth > cdDate.iMonth
			return false;
		}
	}
	else {
		// iYear > cdDate.iYear
		return false;
	}
}


// make the following comparison: is this.date > parameter.date
bool
CalendarDate::bIsGreaterThan( CalendarDate cdDate ) {
	if ( iYear > cdDate.iYear ) {
		return true;
	}
	else if ( iYear == cdDate.iYear ) {
		if ( iMonth > cdDate.iMonth ) {
			return true;
		}
		else if ( iMonth == cdDate.iMonth ) {
			if ( iDay > cdDate.iDay ) {
				return true;
			}
			else {
				// iDay <= cdDate.iDay
				return false;
			}
		}
		else {
			// iMonth < cdDate.iMonth
			return false;
		}
	}
	else {
		// iYear < cdDate.iYear
		return false;
	}
}


// make the following comparison: is this.date == parameter.date
bool
CalendarDate::bIsEqualTo( CalendarDate cdDate ) {
	if ( ( iYear == cdDate.iYear )  &&
		 ( iMonth == cdDate.iMonth )  &&
		 ( iDay == cdDate.iDay ) ) {
		return true;
	}
	else {
		return false;
	}
}


// return the string conversion of the date
char *
CalendarDate::cpDisplayDate() {
	return cpStringDate;
}


// assign the values of an existing calendar date to this calendar date
void
CalendarDate::zSetCalendarDate( CalendarDate cdDate ) {
	iYear = cdDate.iYear;
	iMonth = cdDate.iMonth;
	iDay = cdDate.iDay;
	zConvertToStringDate();
}


// count the number of days (assuming 365 per year, 28 days in February)
// between an interval of two dates
long
CalendarDate::slCountDaysInInterval( CalendarDate cdStart,
									   CalendarDate cdEnd ) {

	long lTotalDays;
	int iThisMonth, iThisYear, iMaxMonth;

	// get days up to end of month
	lTotalDays = I1D_NUMBER_OF_DAYS_IN_MTH[ cdStart.iMonth ]
		- cdStart.iDay;

	if ( ( cdStart.iYear != cdEnd.iYear )  &&
		( cdStart.iMonth != cdEnd.iMonth ) ) {

		iThisMonth = cdStart.iMonth + 1;


		// count days in months 
		for ( iThisYear = cdStart.iYear; iThisYear <= cdEnd.iYear; 
				iThisYear++ ) {

			if ( iThisYear < cdEnd.iYear ) {
				iMaxMonth = 12;
			}
			else {
				iMaxMonth = cdEnd.iMonth;
			}

			while ( iThisMonth < cdEnd.iMonth ) {
				lTotalDays += I1D_NUMBER_OF_DAYS_IN_MTH[ iThisMonth ];
				iThisMonth++;
			}

			iThisMonth = 1;
		}

		// add remaining days
		lTotalDays += cdEnd.iDay;
	}

	return lTotalDays;
}


// conversion of date values to a string
void
CalendarDate::zConvertToStringDate() {
	sprintf( cpStringDate, "%02d/%02d/%02d", iYear, iMonth, iDay );
}


