/* 

Allan Spale
CS 488
Assignment 2

This class will read the landscape, city, and quake data files for the 
3-D earthquake visualization.

*/


#include "DataManagement.h"

// buffer for reading from file
const int DataManagement::KI_BUFFER_SIZE = 100;

//----------------- PUBLIC FUNCTIONS -------------------------------//

DataManagement::DataManagement() {
	lQuakeDataSize = 10;
	lLandscapeDataSize = 10;
	iCityLocationSize = 10;
	
	iLandscapeStructSize = sizeof( LandscapeLocation );
	iQuakeStructSize = sizeof( QuakeData );
	iCityLocationStructSize = sizeof( CityLocation );

	llpLandscape = new LandscapeLocation[ lLandscapeDataSize ];
	qdpQuakeData = new QuakeData[ lQuakeDataSize ];
	clpCities = new CityLocation[ iCityLocationSize ];
}

		
DataManagement::~DataManagement() {
	delete [] llpLandscape;
	delete [] qdpQuakeData;
	delete [] clpCities;
}


// returns a XYZValueer to the earthquake data
QuakeData *
DataManagement::qdpGetEarthquakeData() {
	QuakeData *qdpFilteredData, *qdpTempFilteredData;
	long lIndex, lDataSize, lMaximumDataSizeArray;

	lMaximumDataSizeArray = 10;
	qdpFilteredData = new QuakeData[ lMaximumDataSizeArray ];
	lDataSize = 0;

	for ( lIndex = 0; lIndex < lQuakeDataSize; lIndex++ ) {
		
		memcpy( &qdpFilteredData[ lDataSize ], &qdpQuakeData[ lIndex ],
			    iQuakeStructSize );
			
		lDataSize++;

		// if there is no more room in the array,
		// double the size and copy the old space into the 
		// new space
		if ( lDataSize == lMaximumDataSizeArray ) {

			qdpTempFilteredData = 
				new QuakeData[ lQuakeDataSize * 2 ]; 

			memcpy( qdpTempFilteredData, qdpFilteredData,
					iQuakeStructSize * lMaximumDataSizeArray );
			
			delete [] qdpFilteredData;
			lMaximumDataSizeArray *= 2;
			qdpFilteredData = qdpTempFilteredData;
		}		
	}

	// trim unused space
	qdpTempFilteredData = new QuakeData[ lDataSize ];
	lQuakeDataSize = lDataSize;

	memcpy( qdpTempFilteredData, qdpFilteredData, 
			iQuakeStructSize * lDataSize );
			
	delete [] qdpFilteredData;
	qdpFilteredData = qdpTempFilteredData;

	return qdpFilteredData;
}


// returns a XYZValue to the landscape data
LandscapeLocation * 
DataManagement::llpGetLandscapeData() {
	LandscapeLocation *llLandscapeDataCopy;

	llLandscapeDataCopy = new LandscapeLocation[ lLandscapeDataSize ];
	memcpy( llLandscapeDataCopy, llpLandscape, 
			lLandscapeDataSize * iLandscapeStructSize );

	return llLandscapeDataCopy;
}


CityLocation *
DataManagement::clpGetCityData() {
	CityLocation *clpCitiesCopy;

	clpCitiesCopy = new CityLocation[ iCityLocationSize ];
	memcpy( clpCitiesCopy, clpCities, 
			iCityLocationSize * iCityLocationStructSize );

	return clpCitiesCopy;
}




// read the data files
// NOTE: The method of checking for a non-existent file failed on a
// WindowsXP PC where development was done.  Every time a check
// was made for a file that did not exist, the non-existent file
// was created though no specific request was done so.  However,
// an attempt was made to use the appropriate functions for testing
// for a non-existent file.
 
bool
DataManagement::bReadDataFiles( const char *kcpLandscapeFilename,
								const char *kcpDataFilename,
								const char *kcpCityFilename ) {

	char c1dBuffer[ KI_BUFFER_SIZE ], cpTemp[ 10 ], cpItem[3];
	bool bValidLandscapeFile, bValidDataFile, bValidCitiesFile, bValidFiles;
	long lIndex;
	LandscapeLocation *llpTempLandscape;
	CityLocation *clpTempCityLocation;
	QuakeData *qdpTempQuakeData;
	XYZValue xyzOrigin, xyzMin, xyzMax;

	
	xyzMax.fY = 0.0;
	xyzMax.fX = 0.0;
	xyzMax.fZ = 0.0;
	xyzMin.fY = 99999.0;
	xyzMin.fX = 99999.0;
	xyzMin.fZ = 99999.0;


	strcpy( cpTemp, "         " );
	strcpy( cpItem, "  " );

	cout << "Trying to open landscape file: " << kcpLandscapeFilename << endl;
	cout << "Trying to open earthquake file: " << kcpDataFilename << endl;
	cout << "Trying to open cities file: " << kcpCityFilename << endl << endl;

	// check for the files could be opened; to prevent the 
	// "short-circuit" of one error preventing both errors
	// from being shown, both files will be tested for
	// validity even if the first tested file fails
	ifstream ifLandscapeFile( kcpLandscapeFilename, ios::in );
	if ( !ifLandscapeFile ) {
		cout << "Could not open the landscape file named \""
			 << kcpLandscapeFilename << "\"" << endl << endl;
		bValidLandscapeFile = false;
	}
	else {
		bValidLandscapeFile = true;
	}

	ifstream ifQuakeFile( kcpDataFilename, ios::in );
	if ( !ifQuakeFile ) {
		cout << "Could not open the earthquake data file named \""
			 << kcpDataFilename << "\"" << endl << endl;
		bValidDataFile = false;
	}
	else {
		bValidDataFile = true;
	}

	ifstream ifCitiesFile( kcpCityFilename, ios::in );
	if ( !ifQuakeFile ) {
		cout << "Could not open the cities file named \""
			 << kcpCityFilename << "\"" << endl << endl;
		bValidCitiesFile = false;
	}
	else {
		bValidCitiesFile = true;
	}


	// read data from the files and enter into appropriate arrays
	bValidFiles = bValidLandscapeFile && bValidDataFile && bValidCitiesFile;

	if ( bValidFiles ) {

		//-------------- LANDSCAPE DATA INPUT -------------------//
		// get landscape data first...and ignore the first line
		ifLandscapeFile.getline( c1dBuffer, KI_BUFFER_SIZE );
		lIndex = 0;

		// assuming format: latitude, longitude
		while ( ifLandscapeFile.getline( c1dBuffer, KI_BUFFER_SIZE ) ) {
			if ( strchr( c1dBuffer, ',' ) != NULL ) {
				llpLandscape[ lIndex ].fLatitude = 
					atof( strtok( c1dBuffer, "," ) );

				llpLandscape[ lIndex ].fLongitude = 
					atof( strtok( NULL, "," ) );

				llpLandscape[ lIndex ].bLastPoint = false;

				// check minimum and maximum values for 
				// latitude and longitude
				if ( llpLandscape[ lIndex ].fLatitude > xyzMax.fY ) {
					xyzMax.fY = llpLandscape[ lIndex ].fLatitude;
				}
				else if ( llpLandscape[ lIndex ].fLatitude < xyzMin.fY ) {
					xyzMin.fY = llpLandscape[ lIndex ].fLatitude;
				}

				if ( llpLandscape[ lIndex ].fLongitude > xyzMax.fX ) {
					xyzMax.fX = llpLandscape[ lIndex ].fLongitude;
				}
				else if ( llpLandscape[ lIndex ].fLongitude < xyzMin.fX ) {
					xyzMin.fX = llpLandscape[ lIndex ].fLongitude;
				}

				lIndex++;
				// if there is no more room in the array,
				// double the size and copy the old space into the 
				// new space
				
				if ( lIndex == lLandscapeDataSize ) {

					llpTempLandscape = 
						new LandscapeLocation[ lLandscapeDataSize * 2 ]; 

					memcpy( llpTempLandscape, llpLandscape,
							iLandscapeStructSize * lLandscapeDataSize );
				
					delete [] llpLandscape;
					lLandscapeDataSize *= 2;
					llpLandscape = llpTempLandscape;
				}
			}
			else {
				// mark as last vertex in polygon
				if ( lIndex != 0 ) {
					lIndex--;  // go back to make mark 
					llpLandscape[ lIndex ].bLastPoint = true;
					lIndex++;  // return to current position
				}
			}
		} // end while

		// last point in list is last point for a polygon
		llpLandscape[ lIndex - 1 ].bLastPoint = true;

		// trim unused space
		lLandscapeDataSize = lIndex;

		llpTempLandscape = 
			new LandscapeLocation[ lLandscapeDataSize ]; 

		memcpy( llpTempLandscape, llpLandscape,
				iLandscapeStructSize * lLandscapeDataSize );
			
		delete [] llpLandscape;
		llpLandscape = llpTempLandscape;


		//-------------- QUAKE DATA INPUT -------------------//
		// get landscape data first...and ignore the first line
		ifQuakeFile.getline( c1dBuffer, KI_BUFFER_SIZE );
		lIndex = 0;

		// assuming format: Year, Month, Day, Time(hhmmss.mm)UTC,
		//				    Latitude, Longitude, Magnitude, Depth

		while ( ifQuakeFile.getline( c1dBuffer, KI_BUFFER_SIZE ) ) {
			// if not blank line, process data
			if ( strchr( c1dBuffer, ',' ) != NULL ) {
			
				qdpQuakeData[ lIndex ].cdDate.iYear = 
					atoi( strtok( c1dBuffer, "," ) );

				qdpQuakeData[ lIndex ].cdDate.iMonth = 
					atoi( strtok( NULL, "," ) );

				qdpQuakeData[ lIndex ].cdDate.iDay = 
					atoi( strtok( NULL, "," ) );

				// time processing
				strcpy( cpTemp, strtok( NULL, "," ) );
			
				qdpQuakeData[ lIndex ].tutcTime.iHour =
					atoi( strncpy( cpItem, cpTemp, 2 ) );

				qdpQuakeData[ lIndex ].tutcTime.iMinute =
					atoi( strncpy( cpItem, cpTemp + 2, 2 ) );

				qdpQuakeData[ lIndex ].tutcTime.iSecond =
					atoi( strncpy( cpItem, cpTemp + 4, 2 ) );

				qdpQuakeData[ lIndex ].tutcTime.iMillisecond =
					atoi( strncpy( cpItem, cpTemp + 7, 2 ) );

				// process remainder of line of data but
				
				// OK to convert from double to float...ignore warnings
				qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fY = 
					atof( strtok( NULL, "," ) );

				qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fX = 
					atof( strtok( NULL, "," ) );

				qdpQuakeData[ lIndex ].fMagnitude = 
					atof( strtok( NULL, "," ) );
				qdpQuakeData[ lIndex ].fMagnitudeGreenColor = 
					( 8.0 - qdpQuakeData[ lIndex ].fMagnitude ) / 5.0;

				qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fZ = 
					atof( strtok( NULL, "," ) );

				// do not keep data that has no magnitude
				if ( qdpQuakeData[ lIndex ].fMagnitude > 0.0 ) {

					// check minimum and maximum values for 
					// latitude and longitude and depth
					if ( qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fY > 
							xyzMax.fY ) {
						xyzMax.fY = 
							qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fY;
					}
					else if ( qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fY < 
							xyzMin.fY ) {
						xyzMin.fY = 
							qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fY;
					}

					if ( qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fX >
							xyzMax.fX ) {
						xyzMax.fX = 
							qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fX;
					}
					else if ( qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fX < 
						xyzMin.fX ) {
						xyzMin.fX = 
							qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fX;
					}

					if ( qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fZ >
							xyzMax.fZ ) {
						xyzMax.fZ = 
							qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fZ;
					}
					else if ( qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fZ < 
						xyzMin.fZ ) {
						xyzMin.fZ = 
							qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fZ;
					}
					
					lIndex++;
				}


				// if there is no more room in the array,
				// double the size and copy the old space into the 
				// new space
				
				if ( lIndex == lQuakeDataSize ) {

					qdpTempQuakeData = 
						new QuakeData[ lQuakeDataSize * 2 ]; 

					memcpy( qdpTempQuakeData, qdpQuakeData,
							iQuakeStructSize * lQuakeDataSize );
					delete [] qdpQuakeData;
					lQuakeDataSize *= 2;
					qdpQuakeData = qdpTempQuakeData;
				}
			}  // end if
			// else, go to next line

		} // end while

		// trim unused space
		lQuakeDataSize = lIndex;
		qdpTempQuakeData = 
			new QuakeData[ lQuakeDataSize ]; 

		memcpy( qdpTempQuakeData, qdpQuakeData,
				iQuakeStructSize * lQuakeDataSize );
			
		delete [] qdpQuakeData;
		qdpQuakeData = qdpTempQuakeData;


		//------------ CITY NAME/LOCATION FILE ----------------//
		// get landscape data first...and ignore the first line
		ifCitiesFile.getline( c1dBuffer, KI_BUFFER_SIZE );
		lIndex = 0;

		// assuming format: latitude, longitude
		while ( ifCitiesFile.getline( c1dBuffer, KI_BUFFER_SIZE ) ) {
			clpCities[ lIndex ].llLocation.fLatitude = 
				atof( strtok( c1dBuffer, "," ) );

			clpCities[ lIndex ].llLocation.fLongitude = 
				atof( strtok( NULL, "," ) );

			strcpy( clpCities[ lIndex ].c1dCityName, 
				strtok( NULL, "," ) );

			lIndex++;

			// if there is no more room in the array,
			// double the size and copy the old space into the 
			// new space
			if ( lIndex == iCityLocationSize ) {

				clpTempCityLocation = 
					new CityLocation[ iCityLocationSize * 2 ]; 

				memcpy( clpTempCityLocation, clpCities,
						iCityLocationStructSize * iCityLocationSize );
			
				delete [] clpCities;
				iCityLocationSize *= 2;
				clpCities = clpTempCityLocation;
			}
		} // end while

		// trim unused space
		iCityLocationSize = lIndex;

		clpTempCityLocation = 
			new CityLocation[ iCityLocationSize ]; 

		memcpy( clpTempCityLocation, clpCities,
				iCityLocationStructSize * iCityLocationSize );
			
		delete [] clpCities;
		clpCities = clpTempCityLocation;


		// calculate origin
		xyzOrigin.fX = ( xyzMax.fX + xyzMin.fX ) / 2;
		xyzOrigin.fY = ( xyzMax.fY + xyzMin.fY ) / 2;
		xyzOrigin.fZ = ( xyzMax.fZ + xyzMin.fZ ) / 2;

		zTranslateDataToOrigin( xyzOrigin );
		
	}  // end file processing
	
	return bValidFiles;
}


// return size of quake file 
long 
DataManagement::lGetQuakeDataSize() {
	return lQuakeDataSize;
}


// return size of landscape data file
long 
DataManagement::lGetLandscapeDataSize() {
	return lLandscapeDataSize;
}


// return size of city data file
int
DataManagement::iGetCityDataSize() {
	return iCityLocationSize;
}


// take the real-world coordinates of the data to its origin
void
DataManagement::zTranslateDataToOrigin( XYZValue xyzCenter ) {

	XYZValue xyzTranslationToOrigin;
	long lIndex;

	// find additive inverse of world origin to move to physical origin
	xyzTranslationToOrigin.fX = -xyzCenter.fX;
	xyzTranslationToOrigin.fY = -xyzCenter.fY;
	fZMax = xyzCenter.fZ * 2;

	// landscape data
	for ( lIndex = 0; lIndex < lLandscapeDataSize; lIndex++ ) {
		llpLandscape[ lIndex ].fLongitude += xyzTranslationToOrigin.fX;
		llpLandscape[ lIndex ].fLatitude += xyzTranslationToOrigin.fY;
	}

	// earthquake data
	for ( lIndex = 0; lIndex < lQuakeDataSize; lIndex++ ) {
		qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fX += 
			xyzTranslationToOrigin.fX;
		qdpQuakeData[ lIndex ].qlLocation.xyzLocation.fY += 
			xyzTranslationToOrigin.fY;	
	}

	for ( lIndex = 0; lIndex < iCityLocationSize; lIndex++ ) {
		clpCities[ lIndex ].llLocation.fLongitude += xyzTranslationToOrigin.fX;
		clpCities[ lIndex ].llLocation.fLatitude += xyzTranslationToOrigin.fY;
	}

}


// return the maximum depth of the earthquake data
float
DataManagement::fGetMaximumDepth() {
	return fZMax;
}